@@ -40,6 +40,7 @@ IdSharedMem=str(int(os.getpid()))+"."
 from DDFacet.Other import AsyncProcessPool
 from killMS.Other import ClassFitTEC
 from killMS.Other import ClassFitAmp
+from killMS.Other import ClassClip
 import scipy.ndimage.filters
 from pyrap.tables import table
 # # ##############################
@@ -52,7 +53,8 @@ from pyrap.tables import table
 # warnings.filterwarnings('error')
 # # ##############################
 from killMS.Other.ClassTimeIt import ClassTimeIt
-from killMS.Other.least_squares import least_squares
+#from killMS.Other.least_squares import least_squares
+from scipy.optimize import least_squares
 import copy
 from pyrap.tables import table
 SaveName="last_InterPol.obj"
@@ -108,7 +110,7 @@ class ClassInterpol():
         self.RemoveMedianAmp=RemoveMedianAmp
         
         log.print("Loading %s"%self.InSolsName)
-        self.DicoFile=dict(np.load(self.InSolsName))
+        self.DicoFile=dict(np.load(self.InSolsName,allow_pickle=True))
         self.Sols=self.DicoFile["Sols"].view(np.recarray)
         if "MaskedSols" in self.DicoFile.keys():
             MaskFreq=np.logical_not(np.all(np.all(np.all(self.DicoFile["MaskedSols"][...,0,0],axis=0),axis=1),axis=1))
@@ -235,10 +237,16 @@ class ClassInterpol():
 
         if "PolyAmp" in self.InterpMode:
             for iDir in range(nd):
-                APP.runJob("FitThisPolyAmp_%d"%iJob, self.FitThisPolyAmp, args=(iDir,))#,serial=True)
+                APP.runJob("FitThisPolyAmp_%d"%iJob, self.FitThisPolyAmp, args=(iDir,))
                 iJob+=1
             workers_res=APP.awaitJobResults("FitThisPolyAmp*", progress="Smooth Amp")
 
+        if "Clip" in self.InterpMode:
+            for iDir in range(nd):
+                APP.runJob("ClipThisDir_%d"%iJob, self.ClipThisDir, args=(iDir,),serial=True)
+                iJob+=1
+            workers_res=APP.awaitJobResults("ClipThisDir*", progress="Clip Amp")
+
 
         #APP.terminate()
         APP.shutdown()
@@ -421,7 +429,7 @@ class ClassInterpol():
             return r
 
         #print _f_resid(TEC0CPhase0,A0,A1,ggmeas)
-
+        
         Sol=least_squares(_f_resid,
                           TEC0CPhase0.ravel(),
                           #method="trf",
@@ -484,6 +492,19 @@ class ClassInterpol():
         GOut[:,:,:,iDir,0,0]=gf[:,:,:]
         GOut[:,:,:,iDir,1,1]=gf[:,:,:]
 
+    def ClipThisDir(self,iDir):
+        nt,nch,na,nd,_,_=self.Sols.G.shape
+        GOut=NpShared.GiveArray("%sGOut"%IdSharedMem)
+        # g=GOut[:,:,:,iDir,0,0]
+
+        AmpMachine=ClassClip.ClassClip(self.Sols.G[:,:,:,iDir,0,0],self.CentralFreqs,RemoveMedianAmp=self.RemoveMedianAmp)
+        gf=AmpMachine.doClip()
+        GOut[:,:,:,iDir,0,0]=gf[:,:,:]
+        
+        AmpMachine=ClassClip.ClassClip(self.Sols.G[:,:,:,iDir,1,1],self.CentralFreqs,RemoveMedianAmp=self.RemoveMedianAmp)
+        gf=AmpMachine.doClip()
+        GOut[:,:,:,iDir,1,1]=gf[:,:,:]
+
         
     def FitThisAmp(self,iAnt,iDir):
         nt,nch,na,nd,_,_=self.Sols.G.shape
